@@ -97,7 +97,14 @@ class Squeeze(torch.nn.Module):
         super(Squeeze, self).__init__()
 
     def forward(self, x):
-        return x.squeeze()
+        return x.squeeze(1)
+
+class dim2(torch.nn.Module):
+    def __init__(self, *args):
+        super(dim2, self).__init__()
+
+    def forward(self, x):
+        return  x[:, -1, :]
 
 def cal_accuracy(y, pred_y):
     res = pred_y.argmax(axis=1)
@@ -107,7 +114,8 @@ def cal_accuracy(y, pred_y):
 
 
 class nn:
-    def __init__(self, save_fold=None):
+    def __init__(self, task="cls",save_fold=None):
+        self.task_type = task.lower()
         self.model = torch.nn.Sequential()
         self.batchsize = None
         self.layers = []
@@ -118,14 +126,13 @@ class nn:
         self.res = None
         self.save_fold = "checkpoints"
         self.rnn = False
+        self.input_size = None
         self.color = None # 若载入图片数据集，该变量表示灰度/RGB等颜色通道信息
         self.transform = None # 若载入图片数据集，该变量记录数据预处理操作
         if save_fold != None:
             self.save_fold = save_fold
         self.device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
 
- 
-
     def add(self, layer=None, activation=None, optimizer=None, **kw):
         self.layers_num += 1
         self.layers.append(layer)
@@ -150,7 +157,7 @@ class nn:
             self.model.add_module('maxpooling' + str(self.layers_num), torch.nn.MaxPool2d(kw['kernel_size']))
             print("增加最大池化层,kernel_size: {} ".format(kw['kernel_size']))
         elif layer == 'avgpool':
-            self.model.add_module('maxpooling' + str(self.layers_num), torch.nn.AvgPool2d(kw['kernel_size']))
+            self.model.add_module('avgpooling' + str(self.layers_num), torch.nn.AvgPool2d(kw['kernel_size']))
             print("增加平均池化层,kernel_size: {} ".format(kw['kernel_size']))
         elif layer == 'dropout':
             p = 0.5 if 'p' not in kw.keys() else kw['p']
@@ -186,6 +193,15 @@ class nn:
         elif layer is not None and not isinstance(layer, str):
             print("增加自定义模块{}".format(layer))
             self.model.add_module(str(layer.__class__.__name__)+str(self.layers_num), layer)
+        elif layer =='action_model':
+            l = torch.nn.LSTM(kw['size'][0], kw['size'][1], batch_first=True, bidirectional=False)
+            l = l.to(self.device)
+            self.model.add_module('lstm' + str(self.layers_num),  l)
+            self.model.add_module("part0"+str(self.layers_num), Part0())
+            print("增加lstm层, 输入维度：{}, 输出维度：{},".format(kw['size'][0], kw['size'][1]))
+            self.model.add_module("dim2"+str(self.layers_num), dim2())
+            self.model.add_module('batchnorm1d'+str(self.layers_num),  torch.nn.BatchNorm1d(kw['size'][1]))
+            print("增加batchnorm层, 维度：{}".format(kw['size'][1]))
 
         if activation is not None:
             activation = activation.lower()
@@ -264,7 +280,7 @@ class nn:
 
                     for j in range(tmp_data.shape[0]): # 每一层
                         # print("num+1",num+1, "j", j)
-                        img = tmp_data[j].detach().numpy()
+                        img = tmp_data[j].cpu().detach().numpy()
                         if len(img.shape) == 1:
                             img = np.reshape(img, (img.shape[0],1))
                         # w, c = img.shape
@@ -357,7 +373,7 @@ class nn:
             data = np.squeeze(data)
 
         for i in range(data.shape[0]):
-            img = data[i].detach().numpy()
+            img = data[i].cpu().detach().numpy()
             if len(img.shape) == 1:
                 img = np.reshape(img, (1, img.shape[0]))
             # w, c = img.shape
@@ -374,6 +390,7 @@ class nn:
 
     def load_img_data(self, data_path, classes=None, batch_size=32,train_val_ratio=1.0, shuffle=True,batch_mode=False,transform=None,color="RGB",**kw ):
         from .load_data import ImageFolderDataset
+        self.dataset_type = "img"
         dataset = ImageFolderDataset(data_path, transform=transform,color=color, batch_mode=batch_mode)
         self.dataset_size = int(len(dataset))
         self.color = color
@@ -384,18 +401,32 @@ class nn:
             val_size =  self.dataset_size - train_size
 
             train_dataset, val_dataset = torch.utils.data.random_split(dataset, [train_size, val_size])
-            self.dataloader = DataLoader(train_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
-            self.val_dataloader = DataLoader(val_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
+            self.dataloader = DataLoader(train_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=0)
+            self.val_dataloader = DataLoader(val_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=0)
+            self.input_size = np.array(next(iter(self.dataloader))[0].shape)
+            self.input_size[0] = 1
+
             return self.dataloader, self.val_dataloader
 
         else:
-            self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
-
+            self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=0)
+            self.label = torch.Tensor()
+            for train_x, train_y in self.dataloader:
+                # test_x = test_x.to(self.device)
+                # test_y = test_y.to(self.device)
+                # batch_res = self.model(test_x)
+                self.label = torch.cat([self.label, train_y], dim=0)
+            self.label = self.label.cpu().detach().numpy()
+            self.input_size = np.array(next(iter(self.dataloader))[0].shape)
+            self.input_size[0] = 1
             return self.dataloader
 
-    def load_tab_data(self, data_path,batch_size=32, train_val_ratio=1.0, shuffle=True):
+    def load_tab_data(self, data_path,batch_size=32, train_val_ratio=1.0, shuffle=True,y_type='long'):
         from .load_data import TabularDataset
-        dataset = TabularDataset(data_path)
+        self.dataset_type = "tab"
+        if y_type == 'long' and self.task_type == 'reg':
+            y_type = 'float'
+        dataset = TabularDataset(data_path,y_type=y_type,task_type=self.task_type)
         self.dataset_size = int(len(dataset))
         # print(self.dataset_size)
         # self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
@@ -404,50 +435,49 @@ class nn:
             val_size =  self.dataset_size - train_size
 
             train_dataset, val_dataset = torch.utils.data.random_split(dataset, [train_size, val_size])
-            self.dataloader = DataLoader(train_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
-            self.val_dataloader = DataLoader(val_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
+            self.dataloader = DataLoader(train_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=0)
+            self.val_dataloader = DataLoader(val_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=0)
+            self.input_size = np.array(next(iter(self.dataloader))[0].shape)
+            self.input_size[0] = 1
             return self.dataloader, self.val_dataloader
         else:
-            self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
-
+            self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=0,pin_memory=True)
+            self.input_size = np.array(next(iter(self.dataloader))[0].shape)
+            self.input_size[0] = 1
             return self.dataloader
 
     def load_npz_data(self, data_path, batch_size=32, shuffle=True, classes=None,train_val_ratio=1.0):
         from .load_data import NpzDataset
+        self.dataset_type = "npz"
+
         dataset = NpzDataset(data_path)
         self.dataset_size = int(len(dataset))
-        self.img_classes = classes
-
         # print(self.dataset_size)
-        self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
+        self.img_classes = classes
+        try:
+            self.word2idx = dataset.word2idx
+        except:
+            pass
+        # print(self.dataset_size)
+        # self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
         if 0 < train_val_ratio < 1:
             train_size =  int(train_val_ratio * self.dataset_size)
             val_size =  self.dataset_size - train_size
 
             train_dataset, val_dataset = torch.utils.data.random_split(dataset, [train_size, val_size])
-            self.dataloader = DataLoader(train_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
-            self.val_dataloader = DataLoader(val_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
+            self.dataloader = DataLoader(train_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=0)
+            self.val_dataloader = DataLoader(val_dataset,batch_size=batch_size, shuffle=shuffle,num_workers=0)
+            self.input_size = np.array(next(iter(self.dataloader))[0].shape)
+            self.input_size[0] = 1
         else:
-            self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
+            self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=0)
+            self.input_size = np.array(next(iter(self.dataloader))[0].shape)
+            self.input_size[0] = 1
+
         return self.dataloader
 
-    # def load_dataset(self, data_path, batch_size=32,shuffle=True,batch_mode=False,label_column=-1,transform=None,color="RGB",**kw ):
-        # from .load_data import ImageFolderDataset
-        # from torch.utils.data import DataLoader
-
-        # if data_type == 'tabular':
-        #     dataset = ""
-        # elif data_type =="image_folder":
-        #     dataset = ImageFolderDataset(data_path, transform=transform,color=color, batch_mode=batch_mode)
-        # elif data_type == "custom":
-        #     dataset = ""
-        # else:
-        #     raise ValueError(f"Unsupported data_type: {data_type}")
-        # self.dataset_size = len(dataset)
-        # self.dataloader = DataLoader(dataset,batch_size=batch_size, shuffle=shuffle,num_workers=1)
-        # return self.dataloader
-
     def load_dataset(self, x, y, **kw):
+        self.dataset_type = 'other'
         if kw:
             if 'word2idx'  in kw:
                 print("载入词表")
@@ -484,7 +514,7 @@ class nn:
         self.dataloader = Data.DataLoader(dataset,
                     batch_size = len(dataset),
                     shuffle = True,
-                    num_workers = 1)
+                    num_workers = 0)
 
     def set_seed(self, seed):# 设置随机数种子
         import random
@@ -497,6 +527,10 @@ class nn:
         os.environ['PYTHONHASHSEED'] = str(seed)  # 为了禁止hash随机化，使得实验可复现。
 
     def train(self, lr=0.1, epochs=30, batch_num=1, batch_size=16, save_fold=None, loss="CrossEntropyLoss" ,metrics=["acc"],filename="basenn.pth", checkpoint=None):
+        if self.task_type == 'reg' and 'acc' in metrics:
+            metrics.remove('acc')
+        if  self.task_type == 'reg' and 'mse' not in metrics:
+            metrics.append('mse')
         if self.rnn: # 针对文本任务
             import torch.utils.data as Data
             # from torch.utils.data import DataLoader
@@ -515,7 +549,7 @@ class nn:
             # 设置优化器
             optimizer = torch.optim.Adam(self.model.parameters(), lr = lr)
             # scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optimizer, T_max=150, eta_min=0)
-            scheduler = torch.optim.lr_sheduler.ReduceLROnPlateau(optimizer, mode='min', factor=0.1, patience=10,verbose=False, threshold=0.0001, threshold_mode='rel', cooldown=0, min_lr=0, eps=1e-08)
+            # scheduler = torch.optim.lr_sheduler.ReduceLROnPlateau(optimizer, mode='min', factor=0.1, patience=10,verbose=False, threshold=0.0001, threshold_mode='rel', cooldown=0, min_lr=0, eps=1e-08)
             # 设置损失函数
             criterion = torch.nn.CrossEntropyLoss()
 
@@ -525,21 +559,21 @@ class nn:
 
                     batch_x = batch_x.long().transpose(1, 0).contiguous()
                     batch_y = batch_y.long().transpose(1, 0).contiguous()
-
+                    # print(batch_x.shape, batch_y.shape)
                     batch_x = batch_x.to(self.device)
                     batch_y = batch_y.to(self.device)
                     output, _ = self.model(batch_x)
                     loss = criterion(output, batch_y.view(-1))
 
-                    if batch_idx % 900 == 0:
-                        print('Train Epoch: {} [{}/{} ({:.0f}%)]\tLoss: {:.6f}'.format(
+                    # if batch_idx % 900 == 0:
+                    print('Train Epoch: {} [{}/{} ({:.0f}%)]\tLoss: {:.6f}'.format(
                             epoch+1, batch_idx * len(batch_x[1]), len(self.dataloader.dataset),
                             100. * batch_idx / len(self.dataloader), loss.item()))
 
                     optimizer.zero_grad()
                     loss.backward()
                     optimizer.step()
-                    scheduler.step()
+                    # scheduler.step()
 
             # 保存模型
             print("保存模型中...")
@@ -554,7 +588,8 @@ class nn:
                     'epoch':epochs,
                     'time': time.asctime( time.localtime(time.time()) ),
                     'word2idx': self.word2idx,
-
+                    'input_size':self.input_size,
+                    'dataset_type':self.dataset_type,
                 },
                 'state_dict':self.model.state_dict(),
                 'para':{
@@ -580,7 +615,7 @@ class nn:
                 if not os.path.exists(checkpoint):
                     print("未找到{}文件！".format(checkpoint))
                     return 
-                self.model = torch.load(checkpoint)['state_dict']
+                self.model = torch.load(checkpoint,map_location=torch.device('cpu'))['state_dict']
             # import torch.utils.data as Data
             # dataset = Data.TensorDataset(self.x, self.y)
             # total = self.x.shape[0]
@@ -611,6 +646,8 @@ class nn:
                 # b_loss = 0
                 # b_acc = 0
                 for batch_x, batch_y in self.loader:
+                    # a = time.time()
+                    # print("1", a)
                     # print("iter ", iter, np.squeeze(batch_x[0]).shape, batch_y[0])
                     # import cv2
                     # cv2.imshow("asd", np.array(np.squeeze(batch_x[0])))
@@ -618,17 +655,17 @@ class nn:
                     # y_pred = self.model(self.x)
                     batch_x = batch_x.to(self.device)
                     batch_y = batch_y.to(self.device)
-
                     optimizer.zero_grad()  # 将梯度初始化为零，即清空过往梯度
                     # print(batch_x.shape,type(batch_x))
                     y_pred = self.model(batch_x)
                     # print(batch_y[0],y_pred[0])
                     # cv2.waitKey(0)
-
                     loss = loss_fun(y_pred, batch_y)
                     loss.backward()  # 反向传播，计算当前梯度
                     optimizer.step()  # 根据梯度更新网络参数\
                     # scheduler.step()
+                    # print("2", a - time.time())
+
                     log_info = "{epoch:%d  Loss:%.4f}" % (epoch, loss)
                     log_dict = {"epoch":epoch, "loss":loss.item()}
                     if "acc" in metrics:
@@ -652,6 +689,7 @@ class nn:
                     log.append(log_dict)
                     # b_acc += acc
                     # b_loss+= loss
+
                 # step+=1
                 # print("{epoch:%d  Loss:%.4f  Accuracy:%.4f}" % (epoch, b_loss/step, b_acc/step),step)
 
@@ -674,6 +712,8 @@ class nn:
                     'time': time.asctime( time.localtime(time.time()) ),
                     'color':self.color,
                     'transform':self.transform, 
+                    'input_size':self.input_size,
+                    'dataset_type':self.dataset_type,
                 },
                 'state_dict':self.model,
                 'para':{
@@ -692,11 +732,11 @@ class nn:
 
     def inference(self, data, show=False, checkpoint=None,hidden=None, label=False):
         if checkpoint is not None:
-            self.rnn = torch.load(checkpoint)['para']['rnn']
+            self.rnn = torch.load(checkpoint,map_location=torch.device('cpu'))['para']['rnn']
         if checkpoint:
-            self.model = torch.load(checkpoint)['state_dict']
+            self.model = torch.load(checkpoint,map_location=torch.device('cpu'))['state_dict']
             try:
-                self.img_classes = torch.load(checkpoint)['meta']['CLASSES']
+                self.img_classes = torch.load(checkpoint,map_location=torch.device('cpu'))['meta']['CLASSES']
             except:
                 pass
         if self.rnn:
@@ -725,14 +765,14 @@ class nn:
                         data  = Variable(torch.tensor(np.array(data)).to(torch.float32)).to(self.device)
                 elif file_type.lower() in ["jpg", "png", "jpeg"]:  # 推理图片
                     # img = cv2.imread(data)
-                    color = torch.load(checkpoint)['meta']['color']
+                    color = torch.load(checkpoint,map_location=torch.device('cpu'))['meta']['color']
                     if color == "grayscale":
                         img = cv2.imread(data,cv2.IMREAD_GRAYSCALE)
                         img = np.expand_dims(img, 2)                 
                     else:
                         img = cv2.imread(data)
                     # img = np.array(Image.open(data))
-                    transform = torch.load(checkpoint)['meta']['transform']
+                    transform = torch.load(checkpoint,map_location=torch.device('cpu'))['meta']['transform']
                     if transform is not None:
                         img = np.array(transform(Image.fromarray(img)))
                     data = torch.from_numpy(img).unsqueeze(0).permute(0,3,1,2).to(torch.float32).to(self.device) 
@@ -754,7 +794,7 @@ class nn:
                     for item in dir_list:
                         img_path = os.path.join(data,item)
                         # img = np.array(Image.open(img_path))
-                        color = torch.load(checkpoint)['meta']['color']
+                        color = torch.load(checkpoint,map_location=torch.device('cpu'))['meta']['color']
                         if color == "grayscale":
                             img = cv2.imread(img_path,cv2.IMREAD_GRAYSCALE)
                             img = np.expand_dims(img, 2)                 
@@ -764,7 +804,7 @@ class nn:
                         # if len(img.shape) == 2:
                         #     # 灰度，加1维度（channel）
                         #     img = np.expand_dims(img, 0)                 
-                        transform = torch.load(checkpoint)['meta']['transform']
+                        transform = torch.load(checkpoint,map_location=torch.device('cpu'))['meta']['transform']
                         if transform is not None:
                             img = np.array(transform(Image.fromarray(img)))
                         img_list.append(img)
@@ -788,10 +828,18 @@ class nn:
                 return res
 
             else:  # 推理numpy数组
+                transform = torch.load(checkpoint,map_location=torch.device('cpu'))['meta']['transform']
+                if transform is not None:
+                    data = np.array(transform(Image.fromarray(data)))
+                # if len(np.array(data).shape) == 3:
+                #     data = torch.from_numpy(data).unsqueeze(0).permute(0,3,1,2).to(torch.float32).to(self.device) 
+                # else:
                 data  = Variable(torch.tensor(np.array(data)).to(torch.float32)).to(self.device)
+
+
             # print(data.shape)
             # data  = Variable(torch.tensor(np.array(data)))
-
+            self.model = self.model.to(self.device)
             self.model.eval()
             with torch.no_grad():
                 res = self.model(data)
@@ -807,7 +855,7 @@ class nn:
         if checkpoint and self.device== torch.device('cpu'):
             self.model.load_state_dict(torch.load(checkpoint,map_location=torch.device('cpu'))[1])
         elif checkpoint:
-            sta = torch.load(checkpoint)['state_dict']
+            sta = torch.load(checkpoint,map_location=torch.device('cpu'))['state_dict']
             self.model.load_state_dict(sta)
 
         output, hidden = self.model(data, hidden)
@@ -817,7 +865,7 @@ class nn:
         # print('模型共{}层'.format(self.layers_num))
         print(self.model)
 
-    def save(self, model_path='basenn.pkl'):
+    def save(self, model_path='basenn.pth'):
         print("保存模型中...")
         torch.save(self.model, model_path)
         print("保存模型{}成功！".format(model_path))
@@ -825,7 +873,7 @@ class nn:
     def load(self,model_path):
         print("载入模型中...")
         # self.model = torch.load(model_path)
-        self.model = torch.load(model_path)['state_dict']
+        self.model = torch.load(model_path,map_location=torch.device('cpu'))['state_dict']
 
         print("载入模型{}成功！".format(model_path))
 
@@ -841,3 +889,129 @@ class nn:
         print("推理结果为：", res)
         return res
     
+    def convert(self, checkpoint=None,out_file="convert_model.onnx",backend="ONNX", opset_version=10):
+        # if checkpoint is not None:
+        ckpt = torch.load(checkpoint,map_location=torch.device('cpu'))
+        model = ckpt['state_dict']
+        dataset_type = ckpt['meta']['dataset_type']        
+        input_size = ckpt['meta']['input_size']
+        # print(input_size, dataset_type)
+        x = torch.randn(tuple(input_size)) # tab iris 10,4 ;  image mn 1,3,128,128 ; 
+        tang = False
+        if dataset_type == "img":
+            with torch.no_grad():
+                torch.onnx.export(model, 
+                    x,
+                    out_file,
+                    opset_version=opset_version,
+                    # do_constant_folding=True,	# 是否执行常量折叠优化
+                    input_names=["input"],	# 输入名
+                    output_names=["output"],	# 输出名
+                    dynamic_axes={"input":{0:"batch_size",2:"width",3:"height"},  # 批处理变量
+                                    "output":{0:"batch_size",2:"width",3:"height"}})
+                print(f'Successfully exported ONNX model: {out_file}')
+
+            with open(out_file.replace(".onnx", ".py"), "w+") as f:
+                gen0 = """
+from XEdu.hub import Workflow as wf
+import numpy as np
+
+# 模型声明
+basenn = wf(task='basenn',checkpoint='{}')
+# 待推理图像，此处仅以随机数组为例，以下为{}张{}通道的{}*{}的图像。
+image = np.random.random({}).astype('{}') # 可替换成您想要推理的图像路径,如 image = 'cat.jpg'
+# 模型推理
+res = basenn.inference(data=image)
+# 标准化推理结果
+result = basenn.format_output(lang="zh")
+
+# 更多用法教程详见：https://xedu.readthedocs.io/zh/master/support_resources/model_convert.html
+"""
+                gen = gen0.format(out_file,  x.shape[0], x.shape[1],x.shape[2], x.shape[3],tuple(x.shape).__str__(),str(x.dtype).split('.')[-1],).strip('\n')
+                # gen = gen0.strip('\n') + out_file + gen1.strip('\n') + tuple(input_size).__str__() +gen2.strip('\n')
+                f.write(gen)
+
+        elif dataset_type == "tab":
+            with torch.no_grad():
+                torch.onnx.export(model, 
+                            x,
+                            out_file,
+                            opset_version=opset_version,
+                            # do_constant_folding=True,	# 是否执行常量折叠优化
+                            input_names=["input"],	# 输入名
+                            output_names=["output"],	# 输出名
+                            dynamic_axes={"input":{0:"batch_size"},  # 批处理变量
+                                            "output":{0:"batch_size"}})
+                print(f'Successfully exported ONNX model: {out_file}')
+            with open(out_file.replace(".onnx", ".py"), "w+") as f:
+                gen0 = """
+from XEdu.hub import Workflow as wf
+import numpy as np
+
+# 模型声明
+basenn = wf(task='basenn',checkpoint='{}')
+# 待推理数据，此处仅以随机二维数组为例，以下为{}个维度为{}的特征
+table = np.random.random({}).astype('{}')
+# 模型推理
+res = basenn.inference(data=table)
+# 标准化推理结果
+result = basenn.format_output(lang="zh")
+
+# 更多用法教程详见：https://xedu.readthedocs.io/zh/master/support_resources/model_convert.html
+"""
+                feature_dim = np.squeeze(np.array(x.shape[1:]))
+                gen = gen0.format(out_file, x.shape[0], feature_dim, tuple(x.shape).__str__(),str(x.dtype).split('.')[-1]).strip('\n')
+                # gen = gen0.strip('\n') + out_file + gen1.strip('\n') + tuple(input_size).__str__() +gen2.strip('\n')
+                f.write(gen)
+        elif dataset_type == 'npz':
+            model = ckpt['meta']['backbone']
+            if ckpt['para']['rnn']:
+                x = torch.randint(10,(1,1))         
+                tang = True   
+            with torch.no_grad():
+                torch.onnx.export(model, # tab iris
+                    x,
+                    out_file,
+                    opset_version=opset_version,
+                    # do_constant_folding=True,	# 是否执行常量折叠优化
+                    input_names=["input"],	# 输入名
+                    output_names=["output"],	# 输出名
+                    dynamic_axes={"input":{0:"batch_size"},  # 批处理变量
+                                    "output":{0:"batch_size"}})
+                print(f'Successfully exported ONNX model: {out_file}')
+            with open(out_file.replace(".onnx", ".py"), "w+") as f:
+                gen0 = """
+from XEdu.hub import Workflow as wf
+import numpy as np
+
+# 模型声明
+basenn = wf(task='basenn',checkpoint='{}')
+# 待推理数组，此处仅以随机数组为例
+data = np.random.random({}).astype('{}') # 可替换成您想要推理的npz文件路径,如 data = 'action.npz',npz文件中至少应该包括一个键：data,其中存储数据信息（数组形式）。
+# 模型推理
+res = basenn.inference(data=data)
+# 标准化推理结果
+result = basenn.format_output(lang="zh")
+
+# 更多用法教程详见：https://xedu.readthedocs.io/zh/master/support_resources/model_convert.html
+"""         
+                gen = gen0.format(out_file, tuple(x.shape).__str__(), str(x.dtype).split('.')[-1]).strip('\n')
+                # gen = gen0.strip('\n') + out_file + gen1.strip('\n') + tuple(input_size).__str__() +gen2.strip('\n')
+                f.write(gen)
+            
+        # 插入信息
+        from onnx import load_model,save_model
+        onnx_model = load_model(out_file)
+        # model_info = {'codebase': 'BaseNN', 'input_size': input_size.tolist()}
+        meta = onnx_model.metadata_props.add()
+        meta.key = 'input_size'
+        meta.value = str(input_size.tolist())
+        meta = onnx_model.metadata_props.add()
+        meta.key = 'dataset_type'
+        meta.value = dataset_type
+        if tang:
+            meta = onnx_model.metadata_props.add()
+            meta.key = 'word2idx'
+            meta.value = str(ckpt['meta']['word2idx'])
+            # print(meta.value)
+        save_model(onnx_model, out_file)
\ No newline at end of file
